#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pthread.h>
#include "gexec_options.h"

static int nprocs_ok(int nprocs)
{
    if (nprocs < 1) {
        fprintf(stderr, "-n option (nprocs) must be >= 1\n");
        return 0;
    }
    return 1;
}

static int prefix_type_ok(int prefix_type)
{
    switch (prefix_type) {
    case GEXEC_NO_PREFIX:
    case GEXEC_VNN_PREFIX:
    case GEXEC_HOST_PREFIX:
    case GEXEC_IP_PREFIX:
        return 1;
    default:
        fprintf(stderr, "-p option (prefix_type) invalid");
        return 0;
    }
}

static int port_ok(int port)
{
    if (port < 1) {
        fprintf(stderr, "-P option (port) must be >= 1\n");
        return 0;
    }
    return 1;
}

static int tree_fanout_ok(int tree_fanout)
{
    if (tree_fanout < 1) {
        fprintf(stderr, "-F option (tree_fanout) must be >= 1\n");
        return 0;
    }
    return 1;
}

static void check_options(gexec_options *opts)
{
    if (opts->nprocs == 0) {
        fprintf(stderr, "Must specify -n option (nprocs)\n");
        exit(0);
    }
}

static void print_usage()
{
    printf("Usage: gexec -n nprocs prog arg1 arg2 ...\n");
}

void gexec_options_init(gexec_options *opts)
{
    opts->spawn_opts.verbose        = GEXEC_DEF_VERBOSE;
    opts->spawn_opts.detatched      = GEXEC_DEF_DETATCHED;
    opts->spawn_opts.port           = GEXEC_DEF_PORT;
    opts->spawn_opts.tree_fanout    = GEXEC_DEF_TREE_FANOUT;
    opts->spawn_opts.prefix_type    = GEXEC_DEF_PREFIX_TYPE;
    opts->nprocs                    = 0;
}

void gexec_options_parse(gexec_options *opts, int argc, char **argv,
                         int *args_index)
{
    int c;
    extern char *optarg;
    static struct option long_options[] =
    {
        { "verbose", no_argument, NULL, 'b' },
        { "detatched", no_argument, NULL, 'd' },
        { "nprocs", required_argument, NULL, 'n' },
        { "prefix-type", required_argument, NULL, 'p' },
        { "port", required_argument, NULL, 'P' },
        { 0, 0, 0, 0 }
    };

    *args_index = 1;
    while ((c = getopt_long (argc, argv, "+vdn:p:P:F:", long_options, 
                             (int *)0)) != EOF) {
        switch (c) {
        case 'v':
            opts->spawn_opts.verbose = 1;
            (*args_index)++;
            break;
        case 'd':
            opts->spawn_opts.detatched = 1;
            (*args_index)++;
            break;
        case 'n':
            opts->nprocs = atoi(optarg);
            if (!nprocs_ok(opts->nprocs))
                exit(0);
            (*args_index) += 2;
            break;
        case 'p':
            opts->spawn_opts.prefix_type = atoi(optarg);
            if (!prefix_type_ok(opts->spawn_opts.prefix_type))
                exit(0);
            (*args_index) += 2;
            break;
        case 'P':
            opts->spawn_opts.port = atoi(optarg);
            if (!port_ok(opts->spawn_opts.port))
                exit(0);
            (*args_index) += 2;
            break;
        case 'F':
            opts->spawn_opts.tree_fanout = atoi(optarg);
            if (!tree_fanout_ok(opts->spawn_opts.tree_fanout))
                exit(0);
            (*args_index) += 2;
            break;
        }
    }
    check_options(opts);
    if (*args_index == argc) {
        print_usage();
        exit(0);
    }
}

void gexec_options_print(gexec_options *opts)
{
    printf("gexec_options:  nprocs       = %d\n", opts->nprocs);
    printf("gexec_options:  verbose      = %d\n", opts->spawn_opts.verbose);
    printf("gexec_options:  detatched    = %d\n", opts->spawn_opts.detatched);
    printf("gexec_options:  port         = %d\n", opts->spawn_opts.port);
    printf("gexec_options:  tree_fanout  = %d\n", opts->spawn_opts.tree_fanout);
    printf("gexec_options:  prefix_type  = %d\n", opts->spawn_opts.prefix_type);
}
